@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper;
 
 /*
- * Copyright (c) 1999, 2010 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2011 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -50,6 +50,16 @@ import java.lang.reflect.Modifier;
  *  the property to 300 as follows (defaults to 2 seconds):
  *  -Dorg.tanukisoftware.wrapper.WrapperStartStopApp.maxStartMainWait=300
  * <p>
+ * By default, the WrapperStartStopApp will tell the Wrapper to exit with an
+ *  exit code of 1 if any uncaught exceptions are thrown in the configured
+ *  main method.  This is good in most cases, but is a little different than
+ *  the way Java works on its own.  Java will stay up and running if it has
+ *  launched any other non-daemon threads even if the main method ends because
+ *  of an uncaught exception.  To get this same behavior, it is possible to
+ *  specify the following system property when launching the JVM (defaults to
+ *  FALSE):
+ *  -Dorg.tanukisoftware.wrapper.WrapperStartStopApp.ignoreMainExceptions=TRUE
+ * <p>
  * It is possible to extend this class but make absolutely sure that any
  *  overridden methods call their super method or the class will fail to
  *  function correctly.  Most users will have no need to override this
@@ -126,6 +136,11 @@ public class WrapperStartStopApp
      */
     private Integer m_mainExitCode;
     
+    /**
+     * True if uncaught exceptions in the user app's main method should be ignored.
+     */
+    private boolean m_ignoreMainExceptions;
+    
     /**
      * Flag used to signify that the start method has completed.
      */
@@ -289,21 +304,34 @@ public class WrapperStartStopApp
         // their app threw.
         t.printStackTrace( m_outError );
 
-        synchronized(this)
+        synchronized( this )
         {
-            if ( m_startComplete )
+            if ( m_ignoreMainExceptions )
             {
-                // Shut down here.
-                WrapperManager.stop( 1 );
-                return; // Will not get here.
+                if ( !m_startComplete )
+                {
+                    // An exception was thrown, but we want to let the application continue.
+                    m_mainComplete = true;
+                    this.notifyAll();
+                }
+                return;
             }
             else
             {
-                // Let start method handle shutdown.
-                m_mainComplete = true;
-                m_mainExitCode = new Integer( 1 );
-                this.notifyAll();
-                return;
+                if ( m_startComplete )
+                {
+                    // Shut down here.
+                    WrapperManager.stop( 1 );
+                    return; // Will not get here.
+                }
+                else
+                {
+                    // Let start method handle shutdown.
+                    m_mainComplete = true;
+                    m_mainExitCode = new Integer( 1 );
+                    this.notifyAll();
+                    return;
+                }
             }
         }
     }
@@ -325,6 +353,8 @@ public class WrapperStartStopApp
         // Decide whether or not to wait for the start main method to complete before returning.
         boolean waitForStartMain = WrapperSystemPropertyUtil.getBooleanProperty(
             WrapperStartStopApp.class.getName() + ".waitForStartMain", false );
+        m_ignoreMainExceptions = WrapperSystemPropertyUtil.getBooleanProperty(
+            WrapperStartStopApp.class.getName() + ".ignoreMainExceptions", false );
         int maxStartMainWait = WrapperSystemPropertyUtil.getIntProperty(
             WrapperStartStopApp.class.getName() + ".maxStartMainWait", 2 );
         maxStartMainWait = Math.max( 1, maxStartMainWait ); 
