@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper.test;
 
 /*
- * Copyright (c) 1999, 2010 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2011 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -286,6 +286,7 @@ public abstract class AbstractActionApp
         }
         else if ( action.equals( "access_violation" ) )
         {
+            // The bug we used to cause this is not in most modern VMs so this is not shown by default.
             WrapperManager.accessViolation();
             
         }
@@ -298,17 +299,12 @@ public abstract class AbstractActionApp
         {
             WrapperManager.appearHung();
             
-        }
-        else if ( action.equals( "appear_orphan" ) )
-        {
-            WrapperManager.appearOrphan();
-            
         }
         else if ( action.equals( "deadlock" ) )
         {
             if ( WrapperManager.isStandardEdition() )
             {
-                System.out.println( Main.getRes().getString( "Creating a 2-object dead lock...") );
+                System.out.println( Main.getRes().getString( "Creating a 2-object deadlock...") );
                 DeadLock.create2ObjectDeadlock();
             }
             else
@@ -571,7 +567,7 @@ public abstract class AbstractActionApp
                         WrapperProcessConfig wpConfig = new WrapperProcessConfig();
                         wpConfig.setDetached( m_childDetached );
                         final WrapperProcess wProcess = WrapperManager.exec( m_childCommand, wpConfig );
-                        System.out.println( Main.getRes().getString( "Launched child with PID={0} : {1}", new Integer( wProcess.getPID() ), m_childCommand ) );
+                        System.out.println( Main.getRes().getString( "Launched child process with PID={0} : {1}", new Integer( wProcess.getPID() ), m_childCommand ) );
                         
                         Thread outRunner = new Thread()
                         {
@@ -678,7 +674,6 @@ public abstract class AbstractActionApp
         System.err.println( Main.getRes().getString( "   nestedexit1              : Calls System.exit(1) within WrapperListener.stop(1) callback" ) );
         System.err.println( Main.getRes().getString( "   stopimmediate1           : Calls WrapperManager.stopImmediate(1)" ) );
         System.err.println( Main.getRes().getString( "  Actions which should cause the Wrapper to restart the JVM:" ) );
-        System.err.println( Main.getRes().getString( "   access_violation         : Calls WrapperManager.accessViolation" ) );
         System.err.println( Main.getRes().getString( "   access_violation_native  : Calls WrapperManager.accessViolationNative()" ) );
         System.err.println( Main.getRes().getString( "   appear_hung              : Calls WrapperManager.appearHung()" ) );
         System.err.println( Main.getRes().getString( "   halt0                    : Calls Runtime.getRuntime().halt(0)" ) );
