@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper.test;
 
 /*
- * Copyright (c) 1999, 2010 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2011 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -143,7 +143,7 @@ public class Main
                     getRes().getString( "Calls WrapperManager.stopImmediate( 0 ) to immediately shutdown the JVM and Wrapper with a success exit code." ) );
             
             buildCommand( panel, gridBag, c, "StopImmediate(1)", "stopimmediate1",
-                    getRes().getString(" Calls WrapperManager.stopImmediate( 1 ) to immediately shutdown the JVM and Wrapper with a failure exir code." ) );
+                    getRes().getString(" Calls WrapperManager.stopImmediate( 1 ) to immediately shutdown the JVM and Wrapper with a failure exit code." ) );
             
             buildCommand( panel, gridBag, c, "StopAndReturn(0)", "stopandreturn0",
                     getRes().getString( "Calls WrapperManager.stopAndReturn( 0 ) to shutdown the JVM and Wrapper with a success exit code." ) );
@@ -163,21 +163,15 @@ public class Main
             buildCommand( panel, gridBag, c, "RestartAndReturn()", "restartandreturn",
                     getRes().getString( "Calls WrapperManager.restartAndReturn() to shutdown the current JVM and start a new one." ) );
             
-            buildCommand( panel, gridBag, c, getRes().getString( "Access Violation" ), "access_violation",
-                    getRes().getString( "Attempts to cause an access violation within the JVM, relies on a JVM bug and may not work." ) );
-            
             buildCommand( panel, gridBag, c, getRes().getString( "Native Access Violation" ), "access_violation_native",
                     getRes().getString( "Causes an access violation using native code, the JVM will crash and be restarted." ) );
             
             buildCommand( panel, gridBag, c, getRes().getString( "Simulate JVM Hang" ), "appear_hung",
                     getRes().getString( "Makes the JVM appear to be hung as viewed from the Wrapper, it will be killed and restarted." ) );
             
-            buildCommand( panel, gridBag, c, getRes().getString( "Create DeadLock" ), "deadlock",
+            buildCommand( panel, gridBag, c, getRes().getString( "Create Deadlock" ), "deadlock",
                     getRes().getString( "Creates two new threads which intentionally go into a DeadLock situation.  (Standard, Professional)" ) );
             
-            buildCommand( panel, gridBag, c, getRes().getString( "Simulate Wrapper Crash" ), "appear_orphan",
-                    getRes().getString( "Makes the JVM appear to have been orphaned to simulate the case where the Wrapper has crashed.  The JVM will shut itself down." ) );
-            
             buildCommand( panel, gridBag, c, getRes().getString("Simulate Out Of Memory" ), "outofmemory",
                     getRes().getString( "Throws an OutOfMemoryError to demonstrate the Trigger feature." ) );
             
@@ -263,7 +257,7 @@ public class Main
             Panel childPanel2 = new Panel();
             childPanel2.setLayout( new BorderLayout() );
             childPanel2.add( childPanel, BorderLayout.WEST );
-            buildCommand( panel, gridBag, c, getRes().getString( "Execute Child" ), "child_exec", childPanel2 );
+            buildCommand( panel, gridBag, c, getRes().getString( "Execute Child Process" ), "child_exec", childPanel2 );
             
             buildCommand( panel, gridBag, c, getRes().getString( "GC" ), "gc", getRes().getString( "Performs a GC sweep." ) );
             
